package Telegram;
use strict;
use warnings;
use Data::Dumper;
use warnings::register;
use Carp ();
use constant DEBUG => $ENV{TELEGRAM_BOTAPI_DEBUG} || 0;
our $VERSION = "0.03";
BEGIN {
        eval "require LWP::UserAgent; 1";
}
sub _dprintf { printf "-T- $_[0]\n", splice @_, 1 }
sub _ddump
{
    my ($varname, $to_dump) = splice @_, -2;
    _dprintf @_ if @_;
}
sub new
{
    my ($class, %settings) = @_;
    exists $settings{token}
        or Carp::croak "ERROR: missing 'token' from \%settings.";
    $settings{force_lwp}
        and require LWP::UserAgent;
    $settings{_agent} = ($settings{force_lwp} or !Mojo::UserAgent->can ("new")) ?
        LWP::UserAgent->new : Mojo::UserAgent->new;
    ($settings{async}  ||= 0) and $settings{_agent}->isa ("LWP::UserAgent")
        and Carp::croak "ERROR: Mojo::UserAgent is required to use 'async'.";
    $settings{api_url} ||= "https://api.telegram.org/bot%s/%s";
    DEBUG && _dprintf "Telegram initialized (v%s), using agent %s %ssynchronously.",
        $VERSION, ref $settings{_agent}, $settings{async} ? "a" : "";
    bless \%settings, $class
}
sub DESTROY {}
sub AUTOLOAD
{
    my $self = shift;
    our $AUTOLOAD;
    (my $method = $AUTOLOAD) =~ s/.*:://; # removes the package name at the beginning
    $self->api_request ($method, @_);
}
sub api_request
{
    my ($self, $method) = splice @_, 0, 2;
    my ($postdata, $async_cb);
    foreach my $arg (@_)
    {
        for (ref $arg)
        {
            ($async_cb = $arg, last) if $_ eq "CODE";
            ($postdata = $arg, last) if $_ eq "HASH";
        }
        last if defined $async_cb and defined $postdata;
    }
    $self->{async} and !defined $async_cb
        and Carp::croak "ERROR: a CODE reference (callback) is required when ",
                        "asynchronous requests are enabled.";
    my @request;
    my $is_lwp = $self->_is_lwp;
    push @request, sprintf ($self->{api_url}, $self->{token}, $method);
    if (defined $postdata)
    {
        if ($is_lwp)
        {
            my $has_file_upload = 0;
            foreach my $k (keys %$postdata)
            {
                if (ref $postdata->{$k} eq "HASH")
                {
                    ++$has_file_upload;
                    my $new_val = [];
                    push @$new_val, delete $postdata->{$k}{file},
                                    delete $postdata->{$k}{filename};
                    exists $postdata->{$k}{content}
                        and push @$new_val, Content => delete $postdata->{$k}{content};
                    push @$new_val, %{$postdata->{$k}};
                    $postdata->{$k} = $new_val;
                }
                elsif (ref $postdata->{$k} eq "ARRAY")
                {
                    Carp::croak "ERROR: the upload of multiple files w/LWP is not supported yet.";
                }
            }
            push @request, Content => $postdata,
                           Content_Type => "multipart/form-data";
        }
        else # Mojo::UserAgent, easy peasy
        {
            push @request, form => $postdata;
        }
    }
    push @request, $async_cb if $self->{async};
    return 1 if $self->{_dry_run};
    DEBUG and _ddump "BEGIN REQUEST to /%s :: %s", $method, scalar localtime,
        PAYLOAD => _hide_token ($self, [ @request ]);
    my $tx = $self->agent->post (@request);
    DEBUG and $self->{async} and
        _dprintf "END REQUEST to /%s (async) :: %s", $method, scalar localtime;
    return $tx if $self->{async};
}

sub agent
{
    shift->{_agent}
}
sub _hide_token
{
    $_[1]->[0] =~ s/\Q$_[0]->{token}\E/XXXXXXXXX/g;
    $_[1]
}
sub _is_lwp
{
    shift->agent->isa ("LWP::UserAgent")
}
1;
